@@ -13,8 +13,10 @@ pub enum ParseError {
     IntegerOverflow,
     /// There was extraneous data in the input.
     ExtraData,
-    /// Elements of a set were not lexicographically sorted
+    /// Elements of a set were not lexicographically sorted.
     InvalidSetOrdering,
+    /// An OPTIONAL DEFAULT was written with a default value.
+    EncodedDefault,
 }
 
 /// The result of a `parse`. Either a successful value or a `ParseError`.
@@ -59,6 +61,10 @@ impl<'a> Parser<'a> {
         Ok(())
     }
 
+    pub(crate) fn clone_internal(&self) -> Parser<'a> {
+        Parser::new(self.data)
+    }
+
     pub(crate) fn peek_u8(&mut self) -> Option<u8> {
         self.data.get(0).copied()
     }
@@ -145,7 +151,7 @@ impl<'a> Parser<'a> {
 
     /// This is an alias for `read_element::<Option<Explicit<T, tag>>>` for use
     /// when MSRV is <1.51.
-    pub fn read_optional_explicit_element<T: SimpleAsn1Readable<'a>>(
+    pub fn read_optional_explicit_element<T: Asn1Readable<'a>>(
         &mut self,
         tag: u8,
     ) -> ParseResult<Option<T>> {
@@ -177,8 +183,8 @@ mod tests {
     use super::Parser;
     use crate::types::Asn1Readable;
     use crate::{
-        BigUint, BitString, Choice1, Choice2, Choice3, IA5String, ObjectIdentifier, ParseError,
-        ParseResult, PrintableString, Sequence, SequenceOf, SetOf, Tlv, UtcTime,
+        BigUint, BitString, Choice1, Choice2, Choice3, Enumerated, IA5String, ObjectIdentifier,
+        ParseError, ParseResult, PrintableString, Sequence, SequenceOf, SetOf, Tlv, UtcTime,
     };
     #[cfg(feature = "const-generics")]
     use crate::{Explicit, Implicit};
@@ -554,6 +560,17 @@ mod tests {
         ]);
     }
 
+    #[test]
+    fn test_enumerated() {
+        assert_parses::<Enumerated>(&[
+            (Ok(Enumerated::new(12)), b"\x0a\x01\x0c"),
+            (
+                Err(ParseError::InvalidValue),
+                b"\x0a\x09\xff\xff\xff\xff\xff\xff\xff\xff\xff",
+            ),
+        ]);
+    }
+
     #[test]
     fn test_parse_sequence() {
         assert_parses::<Sequence<'_>>(&[
@@ -621,7 +638,7 @@ mod tests {
                 (Ok(vec![]), b"\x30\x00"),
                 (Err(ParseError::ShortData), b"\x30\x02\x02\x01"),
             ],
-            |p| p.read_element::<SequenceOf<i64>>()?.collect(),
+            |p| Ok(p.read_element::<SequenceOf<i64>>()?.collect()),
         )
     }
 
@@ -644,7 +661,7 @@ mod tests {
                     b"\x31\x02\x01\x00",
                 ),
             ],
-            |p| p.read_element::<SetOf<u64>>()?.collect(),
+            |p| Ok(p.read_element::<SetOf<u64>>()?.collect()),
         )
     }
 
@@ -680,6 +697,12 @@ mod tests {
             (Ok(None), b""),
             (Err(ParseError::ShortData), b"\x04"),
         ]);
+
+        assert_parses::<Option<Choice2<u64, bool>>>(&[
+            (Ok(None), b""),
+            (Ok(Some(Choice2::ChoiceA(17))), b"\x02\x01\x11"),
+            (Ok(Some(Choice2::ChoiceB(true))), b"\x01\x01\xff"),
+        ]);
     }
 
     #[test]
