@@ -15,6 +15,7 @@ pub(crate) const CONSTRUCTED: u8 = 0x20;
 /// Any type that can be parsed as DER ASN.1.
 pub trait Asn1Readable<'a>: Sized {
     fn parse(parser: &mut Parser<'a>) -> ParseResult<Self>;
+    fn can_parse(tag: u8) -> bool;
 }
 
 /// Types with a fixed-tag that can be parsed as DER ASN.1
@@ -25,13 +26,19 @@ pub trait SimpleAsn1Readable<'a>: Sized {
 }
 
 impl<'a, T: SimpleAsn1Readable<'a>> Asn1Readable<'a> for T {
+    #[inline]
     fn parse(parser: &mut Parser<'a>) -> ParseResult<Self> {
         let tlv = parser.read_tlv()?;
-        if tlv.tag != Self::TAG {
+        if !Self::can_parse(tlv.tag) {
             return Err(ParseError::UnexpectedTag { actual: tlv.tag });
         }
         Self::parse_data(tlv.data)
     }
+
+    #[inline]
+    fn can_parse(tag: u8) -> bool {
+        tag == Self::TAG
+    }
 }
 
 /// Any type that can be written as DER ASN.1.
@@ -52,6 +59,13 @@ impl<'a, T: SimpleAsn1Writable<'a>> Asn1Writable<'a> for T {
     }
 }
 
+impl<'a, T: SimpleAsn1Writable<'a>> SimpleAsn1Writable<'a> for &T {
+    const TAG: u8 = T::TAG;
+    fn write_data(&self, dest: &mut Vec<u8>) {
+        T::write_data(self, dest)
+    }
+}
+
 /// A TLV (type, length, value) represented as the tag and bytes content.
 /// Generally used for parsing ASN.1 `ANY` values.
 #[derive(Debug, PartialEq, PartialOrd, Clone, Copy)]
@@ -87,6 +101,15 @@ impl<'a> Asn1Readable<'a> for Tlv<'a> {
     fn parse(parser: &mut Parser<'a>) -> ParseResult<Self> {
         parser.read_tlv()
     }
+
+    fn can_parse(_tag: u8) -> bool {
+        true
+    }
+}
+impl<'a> Asn1Writable<'a> for Tlv<'a> {
+    fn write(&self, w: &mut Writer) {
+        w.write_tlv(self.tag, move |dest| dest.extend_from_slice(self.data))
+    }
 }
 
 impl SimpleAsn1Readable<'_> for () {
@@ -328,6 +351,8 @@ macro_rules! impl_asn1_element_for_int {
 
 impl_asn1_element_for_int!(i8; true);
 impl_asn1_element_for_int!(u8; false);
+impl_asn1_element_for_int!(i32; false);
+impl_asn1_element_for_int!(u32; false);
 impl_asn1_element_for_int!(i64; true);
 impl_asn1_element_for_int!(u64; false);
 
@@ -490,15 +515,47 @@ impl SimpleAsn1Writable<'_> for UtcTime {
     }
 }
 
-impl<'a, T: SimpleAsn1Readable<'a>> Asn1Readable<'a> for Option<T> {
+/// An ASN.1 `ENUMERATED` value.
+#[derive(Debug, PartialEq)]
+pub struct Enumerated(u32);
+
+impl Enumerated {
+    pub fn new(v: u32) -> Enumerated {
+        Enumerated(v)
+    }
+
+    pub fn value(&self) -> u32 {
+        self.0
+    }
+}
+
+impl<'a> SimpleAsn1Readable<'a> for Enumerated {
+    const TAG: u8 = 0xa;
+
+    fn parse_data(data: &'a [u8]) -> ParseResult<Enumerated> {
+        Ok(Enumerated::new(u32::parse_data(data)?))
+    }
+}
+
+impl<'a> SimpleAsn1Writable<'a> for Enumerated {
+    const TAG: u8 = 0xa;
+
+    fn write_data(&self, dest: &mut Vec<u8>) {
+        u32::write_data(&self.0, dest)
+    }
+}
+
+impl<'a, T: Asn1Readable<'a>> Asn1Readable<'a> for Option<T> {
     fn parse(parser: &mut Parser<'a>) -> ParseResult<Self> {
-        let tag = parser.peek_u8();
-        if tag == Some(T::TAG) {
-            Ok(Some(parser.read_element::<T>()?))
-        } else {
-            Ok(None)
+        match parser.peek_u8() {
+            Some(tag) if T::can_parse(tag) => Ok(Some(parser.read_element::<T>()?)),
+            Some(_) | None => Ok(None),
         }
     }
+
+    fn can_parse(tag: u8) -> bool {
+        T::can_parse(tag)
+    }
 }
 
 impl<'a, T: Asn1Writable<'a>> Asn1Writable<'a> for Option<T> {
@@ -509,16 +566,6 @@ impl<'a, T: Asn1Writable<'a>> Asn1Writable<'a> for Option<T> {
     }
 }
 
-impl<'a> Asn1Readable<'a> for Option<Tlv<'a>> {
-    fn parse(parser: &mut Parser<'a>) -> ParseResult<Self> {
-        if parser.is_empty() {
-            Ok(None)
-        } else {
-            Ok(Some(parser.read_element::<Tlv>()?))
-        }
-    }
-}
-
 macro_rules! declare_choice {
     ($count:ident => $(($number:ident $name:ident)),*) => {
         /// Represents an ASN.1 `CHOICE` with the provided number of potential types.
@@ -537,19 +584,28 @@ macro_rules! declare_choice {
         impl<
             'a,
             $(
-                $number: SimpleAsn1Readable<'a>,
+                $number: Asn1Readable<'a>,
             )*
         > Asn1Readable<'a> for $count<$($number,)*> {
             fn parse(parser: &mut Parser<'a>) -> ParseResult<Self> {
                 let tag = parser.peek_u8();
                 match tag {
                     $(
-                        Some(tag) if tag == $number::TAG => Ok($count::$name(parser.read_element::<$number>()?)),
+                        Some(tag) if $number::can_parse(tag) => Ok($count::$name(parser.read_element::<$number>()?)),
                     )*
                     Some(tag) => Err(ParseError::UnexpectedTag{actual: tag}),
                     None => Err(ParseError::ShortData),
                 }
             }
+
+            fn can_parse(tag: u8) -> bool {
+                $(
+                    if $number::can_parse(tag) {
+                        return true;
+                    }
+                )*
+                false
+            }
         }
 
         impl<
@@ -604,6 +660,13 @@ impl<'a> SimpleAsn1Readable<'a> for Sequence<'a> {
         Ok(Sequence::new(data))
     }
 }
+impl<'a> SimpleAsn1Writable<'a> for Sequence<'a> {
+    const TAG: u8 = 0x10 | CONSTRUCTED;
+    #[inline]
+    fn write_data(&self, data: &mut Vec<u8>) {
+        data.extend_from_slice(self.data);
+    }
+}
 
 /// Writes an ASN.1 `SEQUENCE` using a callback that writes the inner
 /// elements.
@@ -643,22 +706,53 @@ impl<'a, T: SimpleAsn1Readable<'a>> SequenceOf<'a, T> {
     }
 }
 
+impl<'a, T: SimpleAsn1Readable<'a>> Clone for SequenceOf<'a, T> {
+    fn clone(&self) -> SequenceOf<'a, T> {
+        SequenceOf {
+            parser: self.parser.clone_internal(),
+            _phantom: PhantomData,
+        }
+    }
+}
+
 impl<'a, T: SimpleAsn1Readable<'a> + 'a> SimpleAsn1Readable<'a> for SequenceOf<'a, T> {
     const TAG: u8 = 0x10 | CONSTRUCTED;
     #[inline]
     fn parse_data(data: &'a [u8]) -> ParseResult<Self> {
+        parse(data, |p| {
+            while !p.is_empty() {
+                p.read_element::<T>()?;
+            }
+            Ok(())
+        })?;
         Ok(SequenceOf::new(data))
     }
 }
 
 impl<'a, T: SimpleAsn1Readable<'a>> Iterator for SequenceOf<'a, T> {
-    type Item = ParseResult<T>;
+    type Item = T;
 
     fn next(&mut self) -> Option<Self::Item> {
         if self.parser.is_empty() {
             return None;
         }
-        Some(self.parser.read_element::<T>())
+        Some(
+            self.parser
+                .read_element::<T>()
+                .expect("Should always succeed"),
+        )
+    }
+}
+
+impl<'a, T: SimpleAsn1Readable<'a> + SimpleAsn1Writable<'a>> SimpleAsn1Writable<'a>
+    for SequenceOf<'a, T>
+{
+    const TAG: u8 = 0x10 | CONSTRUCTED;
+    fn write_data(&self, dest: &mut Vec<u8>) {
+        let mut w = Writer::new(dest);
+        for el in self.clone() {
+            w.write_element(&el);
+        }
     }
 }
 
@@ -687,7 +781,6 @@ impl<'a, T: Asn1Writable<'a>> SimpleAsn1Writable<'a> for SequenceOfWriter<'a, T>
 /// are decoded.
 pub struct SetOf<'a, T: SimpleAsn1Readable<'a>> {
     parser: Parser<'a>,
-    last_element: Option<Tlv<'a>>,
     _phantom: PhantomData<T>,
 }
 
@@ -696,7 +789,6 @@ impl<'a, T: SimpleAsn1Readable<'a>> SetOf<'a, T> {
     pub(crate) fn new(data: &'a [u8]) -> SetOf<'a, T> {
         SetOf {
             parser: Parser::new(data),
-            last_element: None,
             _phantom: PhantomData,
         }
     }
@@ -707,31 +799,36 @@ impl<'a, T: SimpleAsn1Readable<'a> + 'a> SimpleAsn1Readable<'a> for SetOf<'a, T>
 
     #[inline]
     fn parse_data(data: &'a [u8]) -> ParseResult<Self> {
+        parse(data, |p| {
+            let mut last_element: Option<Tlv> = None;
+            while !p.is_empty() {
+                let el = p.read_tlv()?;
+                if let Some(last_el) = last_element {
+                    if el.full_data < last_el.full_data {
+                        return Err(ParseError::InvalidSetOrdering);
+                    }
+                }
+                last_element = Some(el);
+                el.parse::<T>()?;
+            }
+            Ok(())
+        })?;
         Ok(SetOf::new(data))
     }
 }
 
 impl<'a, T: SimpleAsn1Readable<'a>> Iterator for SetOf<'a, T> {
-    type Item = ParseResult<T>;
+    type Item = T;
 
     fn next(&mut self) -> Option<Self::Item> {
         if self.parser.is_empty() {
             return None;
         }
-        let el = match self.parser.read_tlv() {
-            Ok(tlv) => tlv,
-            Err(e) => return Some(Err(e)),
-        };
-        if let Some(last_el) = self.last_element {
-            if el.full_data < last_el.full_data {
-                return Some(Err(ParseError::InvalidSetOrdering));
-            }
-        }
-        self.last_element = Some(el);
-        if el.tag != T::TAG {
-            return Some(Err(ParseError::UnexpectedTag { actual: el.tag }));
-        }
-        Some(T::parse_data(el.data))
+        Some(
+            self.parser
+                .read_element::<T>()
+                .expect("Should always succeed"),
+        )
     }
 }
 
@@ -805,6 +902,13 @@ impl<'a, T, const TAG: u8> Implicit<'a, T, { TAG }> {
     }
 }
 
+#[cfg(feature = "const-generics")]
+impl<'a, T, const TAG: u8> From<T> for Implicit<'a, T, { TAG }> {
+    fn from(v: T) -> Self {
+        Implicit::new(v)
+    }
+}
+
 #[cfg(feature = "const-generics")]
 impl<'a, T: SimpleAsn1Readable<'a>, const TAG: u8> SimpleAsn1Readable<'a>
     for Implicit<'a, T, { TAG }>
@@ -854,9 +958,14 @@ impl<'a, T, const TAG: u8> Explicit<'a, T, { TAG }> {
 }
 
 #[cfg(feature = "const-generics")]
-impl<'a, T: SimpleAsn1Readable<'a>, const TAG: u8> SimpleAsn1Readable<'a>
-    for Explicit<'a, T, { TAG }>
-{
+impl<'a, T, const TAG: u8> From<T> for Explicit<'a, T, { TAG }> {
+    fn from(v: T) -> Self {
+        Explicit::new(v)
+    }
+}
+
+#[cfg(feature = "const-generics")]
+impl<'a, T: Asn1Readable<'a>, const TAG: u8> SimpleAsn1Readable<'a> for Explicit<'a, T, { TAG }> {
     const TAG: u8 = CONTEXT_SPECIFIC | CONSTRUCTED | TAG;
     fn parse_data(data: &'a [u8]) -> ParseResult<Self> {
         Ok(Explicit::new(parse(data, |p| p.read_element::<T>())?))
@@ -864,9 +973,7 @@ impl<'a, T: SimpleAsn1Readable<'a>, const TAG: u8> SimpleAsn1Readable<'a>
 }
 
 #[cfg(feature = "const-generics")]
-impl<'a, T: SimpleAsn1Writable<'a>, const TAG: u8> SimpleAsn1Writable<'a>
-    for Explicit<'a, T, { TAG }>
-{
+impl<'a, T: Asn1Writable<'a>, const TAG: u8> SimpleAsn1Writable<'a> for Explicit<'a, T, { TAG }> {
     const TAG: u8 = CONTEXT_SPECIFIC | CONSTRUCTED | TAG;
     fn write_data(&self, dest: &mut Vec<u8>) {
         Writer::new(dest).write_element(&self.inner);
@@ -875,7 +982,7 @@ impl<'a, T: SimpleAsn1Writable<'a>, const TAG: u8> SimpleAsn1Writable<'a>
 
 #[cfg(test)]
 mod tests {
-    use crate::{IA5String, ParseError, PrintableString, Tlv};
+    use crate::{parse_single, IA5String, ParseError, PrintableString, SequenceOf, Tlv};
 
     #[test]
     fn test_printable_string_new() {
@@ -908,4 +1015,15 @@ mod tests {
             Err(ParseError::UnexpectedTag { actual: 0x2 })
         );
     }
+
+    #[test]
+    fn test_sequence_of_clone() {
+        let mut seq1 =
+            parse_single::<SequenceOf<u64>>(b"\x30\x09\x02\x01\x01\x02\x01\x02\x02\x01\x03")
+                .unwrap();
+        assert_eq!(seq1.next(), Some(1));
+        let seq2 = seq1.clone();
+        assert_eq!(seq1.collect::<Vec<_>>(), vec![2, 3]);
+        assert_eq!(seq2.collect::<Vec<_>>(), vec![2, 3]);
+    }
 }
